/************************************
 * Jacobi Method for Laplace Equation
 * with Dirichlet Boundary Conditions
 * Convergence: biggest change in
 * matrix smaller tolerance value
 * *********************************/

#include <math.h>
#include <string.h>
#include <stdio.h>
#include "realtime.h"
#include "checkResults.h"

#define ROWS 4096
#define COLS 4096

double U[ROWS][COLS];
double Unew[ROWS][COLS];


int main(int argc, char** argv)
{
    int i,j;
    const int n = ROWS, m = COLS;
    const int iter_max = 20;

    const double tol = 1.0e-6;
    double err  = 1.0;
    
    // Initialize arrays (in parallel)
#pragma omp parallel for shared(U,Unew) private(i,j)
    for (i = 0; i < n; i++)
    {
	for (j = 0; j < m; j++)
	{
              U[i][j]    = 0.0;
              Unew[i][j] = 0.0;
	}
    }

        
#pragma omp parallel for shared(Unew, U) private(i)
    for (i = 0; i < n; i++)
    {
        U[0][i]    = -1.0;
        Unew[0][i] = -1.0;
    }

    printf("Solving Laplace Equation by Jacobi Method\n");
    printf("Matrix dim: %d x %d\n\n", n, m);
    printf("Iteration: Error\n");

    double runtime = GetRealTime();
    int iter = 0;    
    
    while ( err > tol && iter < iter_max )
    {
        err = 0.0;

#pragma omp parallel for shared(Unew, U) reduction(max:err) private(i,j)
        for (i = 1; i < n-1; i++)
        {
            for (j = 1; j < m-1; j++)
            {
                Unew[i][j] = 0.25 * ( U[i][j+1] + U[i][j-1]
                                    + U[i-1][j] + U[i+1][j]);
		
		err = fmax(err, fabs(Unew[i][j] - U[i][j]));
            }
        }
        
#pragma omp parallel for shared(Unew, U) private(i,j)
        for (i = 1; i < n-1; i++)
        {
            for (j = 1; j < m-1; j++)
            {
	        U[i][j] = Unew[i][j];    
            }
        }

        iter++;

        printf("%9d: %f\n", iter, err);
    }

    runtime = GetRealTime() - runtime;
/*
    //writeSolution(err, &U[0][0], ROWS, COLS, iter_max); 
    int passed = checkSolution(err, &U[0][0], ROWS, COLS, iter_max);
    if (passed == 0) {
	printf("Error check passed.\n");
    } else if (passed >= 1) {
	printf("Error check failed.\n");
    }
*/
    printf("Time Elapsed: %f s\n", runtime);

    return 0;
}


