module checkResults
    implicit none
    contains

integer function checkSolution(error, A, n, m, iter)
    implicit none
    integer, parameter :: fp_kind=kind(1.0d0)
    real(fp_kind) :: error
    integer :: n,m,iter
    real(fp_kind), dimension (0:n-1,0:m-1) :: A
    integer :: j,i,stat
    real(fp_kind) :: tol, value
    character(len=8) :: str_iter
    character(len=30) :: filename

    tol = 1.0e-16

    write(str_iter,'(i8)') iter
    str_iter=adjustl(str_iter) ! moves string to left hand side
    write(filename,'(A,A,A)') 'solution',trim(str_iter),'.txt'
    !write(*,*) 'Filename:',filename
    open(20, file=filename, iostat=stat)
    if (stat /= 0) then
        write(*,*) 'Error: Solution could not be checked as the solution file could not be read in.'
        checkSolution = 2
        return
    else
        read(20, '(F25.22)', iostat=stat) value
        if (abs(value - error) > tol) then
                write(*,*) 'Error: Results are not correct: Found error: ', error,', correct error: ', value
                checkSolution = 1
                return
        end if
        do j=0,n-1
            do i=0,m-1
                read(20, '(F25.22)', iostat=stat) value
                if (stat /= 0) exit
                if (abs(value - A(j,i)) > tol) then
                   write(*,*) 'Error: Results are not correct. See [j,i] = [',j,',',i,']: found: ',A(j,i),', correct: ',value
                   checkSolution = 1
                   return
                end if
            end do
        end do
        close(20)
    end if
    checkSolution = 0
end function checkSolution

end module checkResults
