#include "cufft_acc.h"

static inline Complex ComplexAdd(Complex, Complex);
static inline Complex ComplexScale(Complex, float);
static inline Complex ComplexMul(Complex, Complex);

////////////////////////////////////////////////////////////////////////////////
// Filtering operations
////////////////////////////////////////////////////////////////////////////////

// Computes convolution on the host
void Convolve(const Complex *signal, int signal_size,
              const Complex *filter_kernel, int filter_kernel_size,
              Complex *filtered_signal)
{
    int i,j;
    int minRadius = filter_kernel_size / 2;
    int maxRadius = filter_kernel_size - minRadius;

    // Loop over output element indices
    for (i = 0; i < signal_size; ++i) {
        filtered_signal[i].x = filtered_signal[i].y = 0;

        // Loop over convolution indices
        for (j = - maxRadius + 1; j <= minRadius; ++j) {
            int k = i + j;

            if (k >= 0 && k < signal_size) {
                filtered_signal[i] = 
                    ComplexAdd(filtered_signal[i], 
                               ComplexMul(signal[k], 
                                          filter_kernel[minRadius - j]));
            }
        }
    }
}

////////////////////////////////////////////////////////////////////////////////
// Complex operations
////////////////////////////////////////////////////////////////////////////////

// Complex addition
static Complex ComplexAdd(Complex a, Complex b)
{
    Complex c;
    c.x = a.x + b.x;
    c.y = a.y + b.y;
    return c;
}

// Complex scale
static inline Complex ComplexScale(Complex a, float s)
{
    Complex c;
    c.x = s * a.x;
    c.y = s * a.y;
    return c;
}

// Complex multiplication
static inline Complex ComplexMul(Complex a, Complex b)
{
    Complex c;
    c.x = a.x * b.x - a.y * b.y;
    c.y = a.x * b.y + a.y * b.x;
    return c;
}

// Complex pointwise multiplication
/*static __global__ void ComplexPointwiseMulAndScale(Complex *a, const Complex *b, int size, float scale)
{
    const int numThreads = blockDim.x * gridDim.x;
    const int threadID = blockIdx.x * blockDim.x + threadIdx.x;

    for (int i = threadID; i < size; i += numThreads) {
        a[i] = ComplexScale(ComplexMul(a[i], b[i]), scale);
    }
}*/

int compareL2(const float* reference, const float* data,
              const unsigned int len, const float epsilon)
{
    unsigned int i;
    double error = 0;
    double ref = 0;

    for(i = 0; i < len; ++i) {
        double diff = reference[i] - data[i];
        error += diff * diff;
        ref += reference[i] * reference[i];
    }

    double normRef = sqrtf(ref);
    double normError = sqrtf(error);
    error = normError / normRef;
    int result = error < epsilon ? 1 : 0;
    if(0 == result)
    {
        printf("ERROR, l2-norm error %e is greater than epsilon %e\n",
               error, epsilon);
    }

    return result;
}
